/* Copyright (c) 2001, Nokia Mobile Phones. All rights reserved */

/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#include <eikgted.h>
#include "SocketsEngine.h"
#include "TimeOutTimer.h"
#include "SocketsRead.h"
#include "SocketsWrite.h"
#include "Sockets.pan"
#include "UINotify.h"
#include "utf.h"


#include <weather.rsg>

static const TInt KTimeOut = 20000000; // 20 seconds time-out



CSocketsEngine* CSocketsEngine::NewL(MHttpNotify& aNotify)
    {
    CSocketsEngine* self = CSocketsEngine::NewLC(aNotify);
    CleanupStack::Pop();
    return self;
    }

    
CSocketsEngine* CSocketsEngine::NewLC(MHttpNotify& aNotify)
    {
    CSocketsEngine* self = new (ELeave) CSocketsEngine(aNotify);
    CleanupStack::PushL(self);
    self->ConstructL();
    return self;
    }


CSocketsEngine::CSocketsEngine(MHttpNotify& aNotify)
: CActive(EPriorityStandard),
  iNotify(aNotify),
  iPort(80)
    {


    }


CSocketsEngine::~CSocketsEngine()
    {
    Cancel();

    delete iSocketsRead;
    iSocketsRead = NULL;

    delete iSocketsWrite;
    iSocketsWrite = NULL;

    delete iTimer;
    iTimer = NULL;

    iSocket.Close();
    iSocketServ.Close();
    }


void CSocketsEngine::ConstructL()
    {
    ChangeStatus(ENotConnected);

    // Start a timer
//  iTimeOut = KTimeOut; 
    iTimer = CTimeOutTimer::NewL(EPriorityHigh, *this);
    CActiveScheduler::Add(this); 

    // Open channel to Socket Server
    User::LeaveIfError(iSocketServ.Connect());

    // Create socket read and write active objects
    iSocketsRead = CSocketsRead::NewL(iNotify, *this, iSocket);
    iSocketsWrite = CSocketsWrite::NewL(iNotify, iSocket);
    }

void CSocketsEngine::ConnectL(TUint32 aAddr) // <a name="ConnectL32">
    {
    // Initiate attempt to connect to a socket by IP address    
    if ((iEngineStatus == ENotConnected) || (iEngineStatus == ETimedOut))
        {
        // Open a TCP socket
        iSocket.Close(); // Just in case
        User::LeaveIfError(iSocket.Open(iSocketServ, KAfInet, KSockStream, KProtocolInetTcp));

        // Set up address information
        iAddress.SetPort(iPort);
        iAddress.SetAddress(aAddr);

        // Initiate socket connection
        iSocket.Connect(iAddress, iStatus);
        ChangeStatus(EConnecting);
        SetActive();

        // Start a timeout
        iTimer->After(KTimeOut);
        }
    }

_LIT8(KHttp10GetFormat, "GET %S HTTP/1.0\r\n\r\n");
//_LIT8(KHttp11GetFormat, "GET %S HTTP/1.1\r\nHost: %S\r\nConnection: close\r\n\r\n");
void CSocketsEngine::StartHttpGetL(const TDesC& aServerName, const TDesC8& aPath, TInt aPort)
    {

    iServerName.Copy(aServerName);
    iHttpRequest.Format(KHttp10GetFormat, &aPath);

/* http 1.1  -- doesn't work yet
    TBuf8<KMaxServerNameLength> serverName8;
    CnvUtfConverter::ConvertFromUnicodeToUtf8(serverName8, aServerName);
    iHttpRequest.Format(KHttp11GetFormat, &aPath, &serverName8);
*/
    iPort = aPort;

    // Initiate connection process
    if ((iEngineStatus == ENotConnected) || (iEngineStatus == ETimedOut))
        {

        iNotify.SocketStatusNotify(R_TEXT_STARTING, EFalse);

        TInetAddr addr;
        if (addr.Input(iServerName) == KErrNone)
            {
            // server name is already a valid ip address
            ConnectL(addr.Address());
            }
        else // need to look up name using dns
            {
            // Initiate DNS
            User::LeaveIfError(iResolver.Open(iSocketServ, KAfInet, KProtocolInetUdp));
            // DNS request for name resolution
            iResolver.GetByName(iServerName, iNameEntry, iStatus);
            
            ChangeStatus(ELookingUp);
            // Request time out
            iTimer->After(KTimeOut);
            SetActive();
            }
        }
    else
        {
        iNotify.SocketStatusNotify(_L("State machine problem!"), EFalse);
        }
    }



void CSocketsEngine::Disconnected()
    {

    ChangeStatus(ENotConnected);

    }


static const TInt KShutdownTimeOut = 2000000; // 2 seconds time-out
void CSocketsEngine::Disconnect()
    {
    iSocketsRead->Cancel();
    iSocketsWrite->Cancel();
    iTimer->Cancel();
    iSocket.Shutdown(RSocket::ENormal, iStatus);
    SetActive();
    iTimer->After(KShutdownTimeOut);
    ChangeStatus(EDisconnecting);
    }

// from CActive
void CSocketsEngine::DoCancel()
    {
    iTimer->Cancel();

    // Cancel appropriate request to socket
    switch (iEngineStatus)
        {
        case EConnecting:
            iSocket.CancelConnect();
            break;
        case ELookingUp:
            // Cancel look up attempt
            iResolver.Cancel();
            iResolver.Close();
            break;
        case EConnected:
            iSocketsRead->Cancel();
            iSocketsWrite->Cancel();
            iSocket.Close();
            break;
        case EDisconnecting:
            iSocket.Close();
            break;
        default:
            User::Panic(KPanicSocketsEngine, ESocketsBadStatus);
            break;
        }

    ChangeStatus(ENotConnected);
    }

void CSocketsEngine::WriteL(const TDesC8& aData)
    {
    iNotify.SocketStatusNotify(R_TEXT_SENDING_REQUEST, EFalse);
    // Write data to socket
    if (iEngineStatus == EConnected)
        {
        iSocketsWrite->IssueWriteL(aData);
        }
    }
    
void CSocketsEngine::Read()
    {

    // Initiate read of data from socket
    if ((iEngineStatus == EConnected) && (!iSocketsRead->IsActive())) 
        {
        iSocketsRead->Start();
        }
    }

// from CActive
void CSocketsEngine::RunL()
    {
    // Active object request complete handler.
    // iEngineStatus flags what request was made, so its
    // completion can be handled appropriately
    iTimer->Cancel(); // Cancel TimeOut timer before completion

    switch(iEngineStatus)
        {
        case EConnecting:
            // IP connection request
            if (iStatus == KErrNone)
            // Connection completed successfully
                {
                RDebug::Print(_L("connected"));
                ChangeStatus(EConnected);

                iSocketsRead->Cancel();

                Read(); //Start CSocketsRead Active object
                
                WriteL(iHttpRequest);
                            
                }
            else
                {
                ChangeStatus(EConnectFailed);
                
                iNotify.SocketStatusNotify(R_TEXT_FAILED_TO_CONNECT, ETrue);
                ChangeStatus(ENotConnected);
                }
            break;
        case ELookingUp:
            iResolver.Close();
            if (iStatus == KErrNone)
                {
                // DNS look up successful
                iNameRecord = iNameEntry();
                // Extract domain name and IP address from name record
                RDebug::Print(_L("\r\nDomain name = "));
                RDebug::Print(iNameRecord.iName);
                TBuf<15> ipAddr;
                TInetAddr::Cast(iNameRecord.iAddr).Output(ipAddr);
                //iConsole.PrintNotify(_L("\r\nIP address = "));
                RDebug::Print(_L("\r\n"));
                // And connect to the IP address
                ChangeStatus(ENotConnected);
                ConnectL(TInetAddr::Cast(iNameRecord.iAddr).Address());
                }
            else
                {   
                // DNS lookup failed
                ChangeStatus(ELookUpFailed);
                iNotify.SocketStatusNotify(R_TEXT_LOOK_UP_FAILED, ETrue);
                ChangeStatus(ENotConnected);
                }
            break;

        case EDisconnecting:
            if (iStatus == KErrNone)
                {
                iSocket.Close();
                ChangeStatus(ENotConnected);
                }
            else
                {
                ChangeStatus(EConnectFailed);
                }
            break;

        default:
            User::Panic(KPanicSocketsEngine, ESocketsBadStatus);
            break;

        };
    }

void CSocketsEngine::TimerExpired()
    {
    Cancel();
    ChangeStatus(ETimedOut);
    //Disconnect();
    }

void CSocketsEngine::ChangeStatus(TSocketsEngineState aNewStatus)
    {
       
    // Update the status (and the status display)
    switch (aNewStatus)
        {
        case ENotConnected:
            //iNotify.SocketStatusNotify(_L("Not connected"), EFalse);
            break;
        case EConnecting:
            iNotify.SocketStatusNotify(R_TEXT_CONNECTING, EFalse);
            break;
        case EConnected:
            iNotify.SocketStatusNotify(R_TEXT_CONNECTED, EFalse);
            break;
        case ETimedOut:
            iNotify.SocketStatusNotify(R_TEXT_TIMED_OUT, ETrue);
            break;
        case ELookingUp:
            iNotify.SocketStatusNotify(R_TEXT_LOOKING_UP, EFalse);
            break;
        case ELookUpFailed:
            iNotify.SocketStatusNotify(R_TEXT_LOOK_UP_FAILED, ETrue);
            break;
        case EConnectFailed:
            iNotify.SocketStatusNotify(R_TEXT_FAILED_TO_CONNECT, ETrue);
            break;
        case EDisconnecting:
            //iNotify.SocketStatusNotify(R_TEXT_DISCONNECTING, ETrue);
            break;
        default:
            User::Panic(KPanicSocketsEngine, ESocketsBadStatus);
            break;
        }

    iEngineStatus = aNewStatus;
    }



TBool CSocketsEngine::Connected() const
    {
    return (iEngineStatus == EConnected);
    }
