
/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


// INCLUDE FILES
#include "weatherview.h"

#include <weather.rsg>
#include "weather.hrh"
#include "weatherappui.h"
#include "weatherutils.h"
#include <eiklabel.h>
#include <gulfont.h>
#include <e32math.h>



CWeatherView::CWeatherView()
    {


    }

// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CWeatherView::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CWeatherView::ConstructL(const TRect& aRect)
    {

    iControlArray = new (ELeave) CArrayPtrFlat<CCoeControl> (5); 


    iMaxRect = aRect;
    iMaxRect.iTl.iY = 0;
    iMaxRect.iBr.iY -= 80;
    SetRect(iMaxRect);


    ActivateL();

    iDenseFont = iEikonEnv->DenseFont();

    TResourceReader reader;
                                            
    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_PHENOMENA);
    iPhenomenonDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iPhenomenonDict
    CleanupStack::PopAndDestroy();    //reader 

    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_MODIFIERS);
    iModifierDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iModifierDict
    CleanupStack::PopAndDestroy();    //reader 

    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_INTENSITY);
    iIntensityDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iIntensityDict
    CleanupStack::PopAndDestroy();    //reader 

    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_CLOUDS);
    iCloudDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iCloudDict
    CleanupStack::PopAndDestroy();    //reader 

    iCoeEnv->CreateResourceReaderLC(reader, R_DESCRIPTION_COMPASS);
    iCompassDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iCompassDict
    CleanupStack::PopAndDestroy();    //reader 


    }


void CWeatherView::AddLabelL(const TDesC& aString, TInt aYPos)
    {

    CEikLabel* label = new (ELeave) CEikLabel;
    label->SetContainerWindowL( *this );
    label->SetFont(iDenseFont);
    label->SetTextL( aString );
    label->SetExtent( TPoint(6,aYPos),  label->MinimumSize());
    iControlArray->AppendL(label);

    }

_LIT(KNoMatchPanic, "No match");
void CWeatherView::AddConditionLabelL(TWeatherCondition& aCondition, TInt aYPos)
    {


    //make string
    HBufC* phenomString = iPhenomenonDict->GetStringForInt(aCondition.iPhenomenon);
    __ASSERT_ALWAYS(phenomString, User::Panic(KNoMatchPanic, 1));
    
    HBufC* intensityString = iIntensityDict->GetStringForInt(aCondition.iIntensity);
    __ASSERT_ALWAYS(intensityString, User::Panic(KNoMatchPanic, 2));
    
    HBufC* modifierString = iModifierDict->GetStringForInt(aCondition.iModifier);
    __ASSERT_ALWAYS(modifierString, User::Panic(KNoMatchPanic, 3));
    
    //phenonString contains the format - e.g. "%S%Swidespread dust"            
    RDebug::Print(*phenomString, intensityString, modifierString);
    
    TBuf<40> description;
    description.Format(*phenomString, intensityString, modifierString);


    TInt bitmapId = -1; //no image

    //choose image
    switch(aCondition.iPhenomenon)
        {
        case EDz:
        case ERa:
            bitmapId = EMbmWeatherRain;
            break;

        case ESn:
        case ESg:
        case EIc:
            bitmapId = EMbmWeatherSnow;
            break;

        case EPe:
        case EGr:
        case EGs:
            bitmapId = EMbmWeatherHail;
            break;

        case EBr:
        case EFg:
        case EHz:
            bitmapId = EMbmWeatherFog;
            break;

        case EVa:
            bitmapId = EMbmWeatherVolcano;
            break;
        default:
            break;

        }

    CBitmapLabel* bitmapLabel = new (ELeave) CBitmapLabel;
    CleanupStack::PushL(bitmapLabel);
    bitmapLabel->ConstructL(iDenseFont, description, bitmapId);
    bitmapLabel->SetContainerWindowL( *this );
    bitmapLabel->SetExtent( TPoint(6,aYPos), TSize(164, 30) );
    iControlArray->AppendL(bitmapLabel);
    CleanupStack::Pop(); //bitmapLabel
    }


void CWeatherView::AddCloudSummaryLabelL(CArrayFixFlat<TCloud>* aClouds, TInt aYPos)
    {

    TInt size;
    if (!aClouds)
        {
        TBuf<80> msgBuf;
        iCoeEnv->ReadResource(msgBuf, R_TEXT_NO_CLOUD_INFORMATION);
        AddLabelL(msgBuf, aYPos);
        return;        
        }

    size = aClouds->Count();

    TCloudCoverage heaviest = EClr;
    TInt i;
    for (i = 0; i < size; i++)
        {
        TCloud cloud = aClouds->At(i);
        if (cloud.iCoverage > heaviest)
            heaviest = cloud.iCoverage;

        }
    

    TInt bitmapId;
    HBufC* cloudSummary;

    switch(heaviest)
        {
        
        case EFew:
        case ESct:
            {
            bitmapId = EMbmWeatherScatteredclouds;
            cloudSummary = iCoeEnv->AllocReadResourceL(R_TEXT_SOME_CLOUDS);
            break;
            }
        case EBkn:
            {
            bitmapId = EMbmWeatherMostlycloudy;
            cloudSummary = iCoeEnv->AllocReadResourceL(R_TEXT_MOSTLY_CLOUDY);
            break;
            }
        case EOvc:
            {
            bitmapId = EMbmWeatherOvercast;
            cloudSummary = iCoeEnv->AllocReadResourceL(R_TEXT_OVERCAST_CLOUDS);
            break;
            }
        default:    //clear skies
            {
            bitmapId = EMbmWeatherSun;
            cloudSummary = iCoeEnv->AllocReadResourceL(R_TEXT_NO_CLOUDS);
            break;
            }
        }

    CBitmapLabel* bitmapLabel = new (ELeave) CBitmapLabel;
    CleanupStack::PushL(bitmapLabel);
    bitmapLabel->ConstructL(iDenseFont, *cloudSummary, bitmapId);
    bitmapLabel->SetContainerWindowL( *this );
    bitmapLabel->SetExtent( TPoint(6,aYPos), TSize(164, 20) );
    iControlArray->AppendL(bitmapLabel);
    CleanupStack::Pop(); //bitmapLabel

    delete cloudSummary;

    }



void CWeatherView::AddCloudLabelL(TCloud& cloud, TInt aYPos)
    {


    HBufC* cloudString = iCloudDict->GetStringForInt(cloud.iCoverage);
    __ASSERT_ALWAYS(cloudString, User::Panic(KNoMatchPanic, 4));


    double heightFraction;   
    TBuf <60> heightFmt;
    TBuf <60> heightString;

    if (GLOBALS->iSettings.iDistanceUnits == EMiles)
        {
        iCoeEnv->ReadResource(heightFmt, R_TEXT_AT_MILES);
        heightFraction = FeetToMiles(cloud.iHeight);
        }
    else    //kilometres
        {
        iCoeEnv->ReadResource(heightFmt, R_TEXT_AT_KILOMETRES);
        heightFraction = ((float)FeetToMetres(cloud.iHeight))/1000;
        }

    heightString.Format(heightFmt, heightFraction);

    RDebug::Print(heightString);

    TBuf<100> description;
    description.Format(*cloudString, &heightString);
       
    RDebug::Print(*cloudString, &heightString);
    RDebug::Print(description);

    TInt bitmapId = -1;

    switch(cloud.iCoverage)
        {
        case EFew:
            bitmapId = EMbmWeatherFewclouds;
            break;
        case ESct:
            bitmapId = EMbmWeatherScatteredclouds;
            break;
        case EBkn:
        case EOvc:
            bitmapId = EMbmWeatherMostlycloudy;
            break;

        default:    //clear skies
            bitmapId = EMbmWeatherSun;
            break;
        }



    CBitmapLabel* bitmapLabel = new (ELeave) CBitmapLabel;
    CleanupStack::PushL(bitmapLabel);
    bitmapLabel->ConstructL(iDenseFont, description, bitmapId);
    bitmapLabel->SetContainerWindowL( *this );
    bitmapLabel->SetExtent( TPoint(6,aYPos), TSize(164, 20) );
    iControlArray->AppendL(bitmapLabel);
    CleanupStack::Pop(); //bitmapLabel

    }


//_LIT(KDateFormat,"%E%D%X%N%Y %1 %2 %3 %H:%T");
_LIT(KDateFormat,"%*E%*D%X%*N%*Y %1 %2 '%3 %H:%T");

_LIT(KNoReportFormat, "No report for %S");


void CWeatherView::DisplayLocation(CLocation* aLocation)
    {

    //clear out any previous controls
    iControlArray->ResetAndDestroy();

    TInt yPos = 5;

    if (!aLocation)
        {
        RDebug::Print(_L("no location"));

        TBuf<80> msgBuf;
        iCoeEnv->ReadResource(msgBuf, R_TEXT_NO_LOCATION_LOADED);

        AddLabelL(msgBuf, yPos);
        return;

        }

    CWeatherInfo* report = aLocation->Report();


    if (!report)
        {
        RDebug::Print(_L("no report for %S"), &(aLocation->Name()));

        TBuf<80> msgBuf;
        TBuf<80> msgFmt;
        iCoeEnv->ReadResource(msgFmt, R_TEXT_NO_REPORT_FOR);
        msgBuf.Format(msgFmt, &(aLocation->Name()));
        AddLabelL(msgBuf, yPos);
        return;
        }

        



    RDebug::Print(_L("report for %S"), &(aLocation->Name()));

/* NOT USED

    TTime reportTime;
    if (report->GetTime(reportTime))
        {
        TBuf<40> dateString;
        reportTime.FormatL(dateString, KDateFormat);
        
        RDebug::Print(_L("date=%S"), &dateString);        

        }

    yPos += 12;
*/






    TInt windSpeed, gustSpeed;
    TCompassDirection direction;
    TBool directionVariable;
    if (report->GetWind(windSpeed, direction, gustSpeed, directionVariable))
        {
        TBuf<10> windFmt; //holds "Wind %S"
        TBuf<40> windString;    //holds the final assembled string
        iCoeEnv->ReadResource(windFmt, R_TEXT_WIND);

        if (windSpeed == 0)
            {
            TBuf<10> calmBuf;
            iCoeEnv->ReadResource(calmBuf, R_TEXT_CALM);
            windString.Format(windFmt, &calmBuf);
            RDebug::Print(_L("Wind: Calm"));
            //AddLabelL( _L("Wind: Calm"), yPos);
            }
        else 
            {

            TBuf<20> buf;
            MakeSpeedString(buf, windSpeed);
            RDebug::Print(_L("windSpeed is %S"), &buf);
            windString.Format(windFmt, &buf);

            if (directionVariable)
                {
                iCoeEnv->ReadResource(buf, R_TEXT_DIRECTION_VARIES);
                RDebug::Print(_L("Direction varies"));
                windString.Append(buf);
                }
            else
                {
                HBufC* compassString = iCompassDict->GetStringForInt(direction);
                __ASSERT_ALWAYS(compassString, User::Panic(KNoMatchPanic, 0));
                RDebug::Print(_L("%S"), compassString);
                windString.Append(*compassString);
                }

            if (gustSpeed > 0) 
                {
                TBuf<10> gustsFmt;
                iCoeEnv->ReadResource(buf, R_TEXT_GUSTS_OF);
                MakeSpeedString(buf, gustSpeed);
                RDebug::Print(_L("gustSpeed is %S"), &buf);
                windString.AppendFormat(gustsFmt, &buf);
                }
            }
            AddLabelL( windString, yPos);
            
        }
    else
        {
        TBuf<40> buf;
        iCoeEnv->ReadResource(buf, R_TEXT_NO_WIND_INFORMATION);
        AddLabelL( buf, yPos);
        }

    yPos += 15;


    TInt visibility;
    if (report->GetVisibility(visibility))
        {

        TBuf<40> visibilityString;

        if (visibility == 9999)
            {
            iCoeEnv->ReadResource(visibilityString, R_TEXT_VISIBILITY_UNLIMITED);
            }
        else
            {
            double visibilityFraction;   
            TBuf <40> visibilityFmt;

            if (GLOBALS->iSettings.iDistanceUnits == EMiles)
                {
                iCoeEnv->ReadResource(visibilityFmt, R_TEXT_VISIBILITY_MILES);
                visibilityFraction = MetresToMiles(visibility);
                }
            else    //kilometres
                {
                iCoeEnv->ReadResource(visibilityFmt, R_TEXT_VISIBILITY_KILOMETRES);
                visibilityFraction = ((float)visibility)/1000;
                }
            visibilityString.Format(visibilityFmt, visibilityFraction);

            }
            AddLabelL(visibilityString, yPos);
            yPos += 15;

        }

   
    
    yPos += 10;

    CArrayFixFlat<TWeatherCondition>* conditions = report->GetConditions();
    if (conditions)
        {
        TInt size = conditions->Count();
        TInt i;
        for (i = 0; i < size; i++)
            {
            TWeatherCondition condition = conditions->At(i);

            AddConditionLabelL(condition, yPos);
            yPos += 30;
            }

        }


    CArrayFixFlat<TCloud>* clouds = report->GetClouds();

    if (clouds)
        {

        if (GLOBALS->iSettings.iDetailedCloudReports)
            {
            TInt size = clouds->Count();
            TInt i;
            for (i = 0; i < size; i++)
                {
                TCloud cloud = clouds->At(i);

                AddCloudLabelL(cloud, yPos);
                yPos += 40;
                }
            }
        else
            {
            AddCloudSummaryLabelL(clouds, yPos);
            yPos += 30;
            }

        }


    TInt millibars;
    if (report->GetPressure(millibars))
        {
        TBuf<40> pressureFmt;
        iCoeEnv->ReadResource(pressureFmt, R_TEXT_PRESSURE_MB);
        RDebug::Print(pressureFmt, millibars);
        TBuf<40> pressureString;
        pressureString.Format(pressureFmt, millibars);

        AddLabelL( pressureString, yPos);

        yPos += 15;
        }

    TInt temperature;
    if (report->GetTemperature(temperature))
        {

        TInt dewPoint;
        if (report->GetDewPoint(dewPoint))
            {

            TBuf<10> tempBuf;
            MakeTempString(tempBuf, dewPoint);
        
            TReal relativeHumidity;
            
            Math::Round(relativeHumidity, (RelativeHumidity((double)temperature, (double)dewPoint)*100), 0);


            TBuf<80> msgBuf;
            TBuf<80> fmt;

            iCoeEnv->ReadResource(fmt, R_TEXT_DEW_POINT_AND_HUMIDITY);
            msgBuf.Format(fmt, &tempBuf, (TInt)relativeHumidity);


            AddLabelL(msgBuf, yPos);

            yPos += 20;

            }


        }



    TRect rect = Rect();
    rect.iTl.iY = 0;
    rect.SetHeight(yPos);
    SetRect(rect);
    RDebug::Print(_L("top left x=%d y=%d  height=%d"), rect.iTl.iX, rect.iTl.iY, rect.Height());

    ActivateL();

    DrawNow();
    }


// Destructor
CWeatherView::~CWeatherView()
    {
    iControlArray->ResetAndDestroy();
    delete iControlArray;

    delete iPhenomenonDict;
    delete iModifierDict;
    delete iIntensityDict;
    delete iCloudDict;
    delete iCompassDict;

    }

// ---------------------------------------------------------
// CWeatherView::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CWeatherView::SizeChanged()
    {
    }

// ---------------------------------------------------------
// CWeatherView::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CWeatherView::CountComponentControls() const
    {
    return iControlArray->Count();

    }

// ---------------------------------------------------------
// CWeatherView::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CWeatherView::ComponentControl(TInt aIndex) const
    {

    return iControlArray->At(aIndex);

    }

// ---------------------------------------------------------
// CWeatherView::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CWeatherView::Draw(const TRect& aRect) const
    {

    RDebug::Print(_L("CWeatherView::Draw"));

    CWindowGc& gc = SystemGc();
    gc.SetPenStyle(CGraphicsContext::ENullPen);
    gc.SetBrushColor(KRgbWhite);
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(aRect);

    }


// End of File  
